// Generated by ResgenSwift.Analytics 1.2

import MatomoTracker
import Firebase

// MARK: - Protocol

protocol AnalyticsManagerProtocol {
    func logScreen(name: String, path: String)
    func logEvent(
        name: String,
        action: String,
        category: String,
        params: [String: Any]?
    )
}

// MARK: - Matomo

class MatomoAnalyticsManager: AnalyticsManagerProtocol {
    
    // MARK: - Properties

    private var tracker: MatomoTracker

    // MARK: - Init

    init(siteId: String, url: String) {
        debugPrint("[Matomo service] Server URL: \(url)")
        debugPrint("[Matomo service] Site ID: \(siteId)")
        tracker = MatomoTracker(
            siteId: siteId,
            baseURL: URL(string: url)!
        )
        
        #if DEBUG
            tracker.dispatchInterval = 5
        #endif
        
        #if DEBUG
            tracker.logger = DefaultLogger(minLevel: .verbose)
        #endif

        debugPrint("[Matomo service] Configured with content base: \(tracker.contentBase?.absoluteString ?? "-")")
        debugPrint("[Matomo service] Opt out: \(tracker.isOptedOut)")
    }
    
    // MARK: - Methods

    func logScreen(name: String, path: String) {
        guard !tracker.isOptedOut else { return }
        guard let trackerUrl = tracker.contentBase?.absoluteString else { return }

        let urlString = URL(string: "\(trackerUrl)" + "/" + "\(path)" + "iOS")
        tracker.track(
            view: [name],
            url: urlString
        )
    }

    func logEvent(
        name: String,
        action: String,
        category: String,
        params: [String: Any]?
    ) {
        guard !tracker.isOptedOut else { return }

        tracker.track(
            eventWithCategory: category,
            action: action,
            name: name,
            number: nil,
            url: nil
        )
    }
}

// MARK: - Firebase

class FirebaseAnalyticsManager: AnalyticsManagerProtocol {
    func logScreen(name: String, path: String) {
        var parameters = [
            AnalyticsParameterScreenName: name
        ]

        Analytics.logEvent(
            AnalyticsEventScreenView,
            parameters: parameters
        )
    }

    func logEvent(
        name: String,
        action: String,
        category: String,
        params: [String: Any]?
    ) {
        var parameters: [String:Any] = [
            "action": action,
            "category": category,
        ]

        if let supplementaryParameters = params {
            parameters.merge(supplementaryParameters) { (origin, new) -> Any in
                return origin
            }
        }
        
        Analytics.logEvent(
            name,
            parameters: parameters
        )
    }
}

// MARK: - Manager

class AnalyticsManager {
    static var shared = AnalyticsManager()
    
    // MARK: - Properties

    var managers: [AnalyticsManagerProtocol] = []

    private var isEnabled: Bool = true

    // MARK: - Methods

    func setAnalyticsEnabled(_ enable: Bool) { 
        isEnabled = enable
    }

    func configure(siteId: String, url: String) {
        managers.append(
            MatomoAnalyticsManager(
                siteId: siteId,
                url: url
            )
        )
        managers.append(FirebaseAnalyticsManager())
    }
        
    private func logScreen(name: String, path: String) {
        guard isEnabled else { return }

        managers.forEach { manager in
            manager.logScreen(name: name, path: path)
        }
    }
    
    private func logEvent(
        name: String,
        action: String,
        category: String,
        params: [String: Any]?
    ) {
        guard isEnabled else { return }

        managers.forEach { manager in
            manager.logEvent(
                name: name,
                action: action,
                category: category,
                params: params
            )
        }
    }

    // MARK: - Introduction

    func logScreenIntroductionScreen(title: String) {
        logScreen(
            name: "Bienvenue \(title)",
            path: "introduction/"
        )
    }

    func logEventIntroductionScreen(test: String, data: Int) {
        logEvent(
            name: "Bienvenue",
            action: "action",
            category: "category",
            params: [
                "test": test,
				"data": data
            ]
        )
    }
}
